@@ -19,10 +19,11 @@ foo
           )
           ( b,
             d
-          ) -<
-          ( b + 1, -- Funnel into arrow
-            d * b
           )
+          -<
+            ( b + 1, -- Funnel into arrow
+              d * b
+            )
       if x `mod` y == 0 -- Basic condition
         then case e of -- Only left case is relevant
           Left
@@ -32,18 +33,20 @@ foo
               let v =
                     u -- Actually never used
                       ^ 2
-               in ( returnA -<
-                      -- Just do the calculation
-                      (x + y * z)
+               in ( returnA
+                      -<
+                        -- Just do the calculation
+                        (x + y * z)
                   )
         else do
           let u = x -- Let bindings bind expressions, not commands
-            -- Could pattern match directly on x
+          -- Could pattern match directly on x
           i <- case u of
             0 -> (g . h -< u)
             n ->
-              ( ( h . g -<
-                    y -- First actual use of y
+              ( ( h . g
+                    -<
+                      y -- First actual use of y
                 )
               )
           returnA -< ()
@@ -51,8 +54,9 @@ foo
           if i > 0
             then ma -< ()
             else returnA -< ()
-          returnA -<
-            ( i
-                + x
-                * y -- Just do the calculation
-            )
+          returnA
+            -<
+              ( i
+                  + x
+                  * y -- Just do the calculation
+              )
