@@ -10,6 +10,3 @@ g ((f, _), f -> 4) = True
 multiline (t -> Foo
                   bar
                   baz) = True
-
--- https://github.com/tweag/ormolu/issues/343
-foo = (f -> 4)
