@@ -11,9 +11,6 @@ import GHC
 import Ormolu.Printer.Combinators
 import Ormolu.Printer.Meat.Common
 import Ormolu.Printer.Meat.Type
-import Ormolu.Utils (notImplemented)
-import RdrName (RdrName (..))
-import SrcLoc (Located)
 
 p_synDecl ::
   -- | Type constructor
@@ -31,11 +28,13 @@ p_synDecl name fixity HsQTvs {..} t = do
   switchLayout (getLoc name : map getLoc hsq_explicit) $
     p_infixDefHelper
       (case fixity of Infix -> True; _ -> False)
-      inci
+      True
       (p_rdrName name)
       (map (located' p_hsTyVarBndr) hsq_explicit)
   space
-  txt "="
-  breakpoint
+  equals
+  if hasDocStrings (unLoc t)
+    then newline
+    else breakpoint
   inci (located t p_hsType)
-p_synDecl _ _ XLHsQTyVars {} _ = notImplemented "XLHsQTyVars"
+p_synDecl _ _ (XLHsQTyVars x) _ = noExtCon x
