@@ -11,10 +11,9 @@ where
 
 import Control.Monad
 import GHC
-import HsImpExp (IE (..))
 import Ormolu.Printer.Combinators
 import Ormolu.Printer.Meat.Common
-import Ormolu.Utils
+import Ormolu.Utils (RelativePos (..), attachRelativePos)
 
 p_hsmodExports :: [LIE GhcPs] -> R ()
 p_hsmodExports [] = do
@@ -22,19 +21,24 @@ p_hsmodExports [] = do
   breakpoint'
   txt ")"
 p_hsmodExports xs =
-  parens N . sitcc $ do
+  parens N $ do
     layout <- getLayout
-    sep breakpoint (sitcc . located' (uncurry (p_lie layout))) (attachPositions xs)
+    sep
+      breakpoint
+      (\(p, l) -> sitcc (located l (p_lie layout p)))
+      (attachRelativePos xs)
 
-p_hsmodImport :: ImportDecl GhcPs -> R ()
-p_hsmodImport ImportDecl {..} = do
+p_hsmodImport :: Bool -> ImportDecl GhcPs -> R ()
+p_hsmodImport useQualifiedPost ImportDecl {..} = do
   txt "import"
   space
   when ideclSource (txt "{-# SOURCE #-}")
   space
   when ideclSafe (txt "safe")
   space
-  when ideclQualified (txt "qualified")
+  when
+    (isImportDeclQualified ideclQualified && not useQualifiedPost)
+    (txt "qualified")
   space
   case ideclPkgQual of
     Nothing -> return ()
@@ -42,6 +46,9 @@ p_hsmodImport ImportDecl {..} = do
   space
   inci $ do
     located ideclName atom
+    when
+      (isImportDeclQualified ideclQualified && useQualifiedPost)
+      (space >> txt "qualified")
     case ideclAs of
       Nothing -> return ()
       Just l -> do
@@ -58,60 +65,62 @@ p_hsmodImport ImportDecl {..} = do
       Nothing -> return ()
       Just (_, L _ xs) -> do
         breakpoint
-        parens N . sitcc $ do
+        parens N $ do
           layout <- getLayout
-          sep breakpoint (sitcc . located' (uncurry (p_lie layout))) (attachPositions xs)
+          sep
+            breakpoint
+            (\(p, l) -> sitcc (located l (p_lie layout p)))
+            (attachRelativePos xs)
     newline
-p_hsmodImport (XImportDecl NoExt) = notImplemented "XImportDecl"
+p_hsmodImport _ (XImportDecl x) = noExtCon x
 
-p_lie :: Layout -> (Int, Int) -> IE GhcPs -> R ()
-p_lie encLayout (i, totalItems) = \case
-  IEVar NoExt l1 -> do
+p_lie :: Layout -> RelativePos -> IE GhcPs -> R ()
+p_lie encLayout relativePos = \case
+  IEVar NoExtField l1 -> do
     located l1 p_ieWrappedName
     p_comma
-  IEThingAbs NoExt l1 -> do
+  IEThingAbs NoExtField l1 -> do
     located l1 p_ieWrappedName
     p_comma
-  IEThingAll NoExt l1 -> do
+  IEThingAll NoExtField l1 -> do
     located l1 p_ieWrappedName
     space
     txt "(..)"
     p_comma
-  IEThingWith NoExt l1 w xs _ -> sitcc $ do
+  IEThingWith NoExtField l1 w xs _ -> sitcc $ do
     located l1 p_ieWrappedName
     breakpoint
     inci $ do
       let names :: [R ()]
           names = located' p_ieWrappedName <$> xs
-      parens N . sitcc
-        $ sep (comma >> breakpoint) sitcc
-        $ case w of
+      parens N . sep commaDel sitcc $
+        case w of
           NoIEWildcard -> names
           IEWildcard n ->
             let (before, after) = splitAt n names
              in before ++ [txt ".."] ++ after
     p_comma
-  IEModuleContents NoExt l1 -> do
+  IEModuleContents NoExtField l1 -> do
     located l1 p_hsmodName
     p_comma
-  IEGroup NoExt n str -> do
-    unless (i == 0) newline
+  IEGroup NoExtField n str -> do
+    case relativePos of
+      SinglePos -> return ()
+      FirstPos -> return ()
+      MiddlePos -> newline
+      LastPos -> newline
     p_hsDocString (Asterisk n) False (noLoc str)
-  IEDoc NoExt str ->
+  IEDoc NoExtField str ->
     p_hsDocString Pipe False (noLoc str)
-  IEDocNamed NoExt str -> p_hsDocName str
-  XIE NoExt -> notImplemented "XIE"
+  IEDocNamed NoExtField str -> p_hsDocName str
+  XIE x -> noExtCon x
   where
     p_comma =
       case encLayout of
-        SingleLine -> unless (i + 1 == totalItems) comma
+        SingleLine ->
+          case relativePos of
+            SinglePos -> return ()
+            FirstPos -> comma
+            MiddlePos -> comma
+            LastPos -> return ()
         MultiLine -> comma
-
--- | Attach positions to 'Located' things in a list.
-attachPositions ::
-  [Located a] ->
-  [Located ((Int, Int), a)]
-attachPositions xs =
-  let f i (L l x) = L l ((i, n), x)
-      n = length xs
-   in zipWith f [0 ..] xs
