@@ -1,3 +1,4 @@
+{-# LANGUAGE LambdaCase #-}
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE RecordWildCards #-}
 
@@ -10,8 +11,10 @@ where
 import Control.Monad
 import qualified Data.Text as T
 import GHC
-import Ormolu.Imports
+import Ormolu.Imports (normalizeImports)
+import Ormolu.Parser.CommentStream
 import Ormolu.Parser.Pragma
+import Ormolu.Parser.Shebang
 import Ormolu.Printer.Combinators
 import Ormolu.Printer.Comments
 import Ormolu.Printer.Meat.Common
@@ -22,26 +25,28 @@ import Ormolu.Printer.Meat.Pragma
 
 -- | Render a module.
 p_hsModule ::
+  -- | Stack header
+  Maybe (RealLocated Comment) ->
   -- | Shebangs
-  [Located String] ->
-  -- | Pragmas
-  [Pragma] ->
+  [Shebang] ->
+  -- | Pragmas and the associated comments
+  [([RealLocated Comment], Pragma)] ->
+  -- | Whether to use postfix qualified imports
+  Bool ->
   -- | AST to print
-  ParsedSource ->
+  HsModule GhcPs ->
   R ()
-p_hsModule shebangs pragmas (L moduleSpan HsModule {..}) = do
-  -- If span of exports in multiline, the whole thing is multiline. This is
-  -- especially important because span of module itself always seems to have
-  -- length zero, so it's not reliable for layout selection.
-  let exportSpans = maybe [] (\(L s _) -> [s]) hsmodExports
-      deprecSpan = maybe [] (\(L s _) -> [s]) hsmodDeprecMessage
-      spans' = exportSpans ++ deprecSpan ++ [moduleSpan]
-  switchLayout spans' $ do
-    forM_ shebangs $ \x ->
+p_hsModule mstackHeader shebangs pragmas qualifiedPost HsModule {..} = do
+  let deprecSpan = maybe [] (\(L s _) -> [s]) hsmodDeprecMessage
+      exportSpans = maybe [] (\(L s _) -> [s]) hsmodExports
+  switchLayout (deprecSpan <> exportSpans) $ do
+    forM_ shebangs $ \(Shebang x) ->
       located x $ \shebang -> do
         txt (T.pack shebang)
         newline
-    spitStackHeader
+    forM_ mstackHeader $ \(L spn comment) -> do
+      spitCommentNow spn comment
+      newline
     newline
     p_pragmas pragmas
     newline
@@ -51,19 +56,20 @@ p_hsModule shebangs pragmas (L moduleSpan HsModule {..}) = do
         located hsmodName' $ \name -> do
           forM_ hsmodHaddockModHeader (p_hsDocString Pipe True)
           p_hsmodName name
+        breakpoint
         forM_ hsmodDeprecMessage $ \w -> do
-          breakpoint
           located' p_moduleWarning w
+          breakpoint
         case hsmodExports of
           Nothing -> return ()
-          Just hsmodExports' -> do
+          Just l -> do
+            located l $ \exports -> do
+              inci (p_hsmodExports exports)
             breakpoint
-            inci (p_hsmodExports (unLoc hsmodExports'))
-        breakpoint
         txt "where"
         newline
     newline
-    forM_ (sortImports hsmodImports) (located' p_hsmodImport)
+    forM_ (normalizeImports hsmodImports) (located' (p_hsmodImport qualifiedPost))
     newline
     switchLayout (getLoc <$> hsmodDecls) $ do
       p_hsDecls Free hsmodDecls
